// Dependencies
import fs from "fs";
import degit from "degit";
import args from "args";
import chalk from "chalk";
import { spawn } from "child_process";

// Command Arguments
args.option("directory", "Name of the directory").examples([
  {
    usage: `typescript-express-api -d my-api`,
    description: `The well defined base express js app in typescript.`,
  },
]);

// Parsing Command Arguments
const flags = args.parse(process.argv);

// Error Logger
const logError = (message) => {
  console.error(chalk.red.bgBlack(message));
};

// System Command Runner
const runCommand = (command, args, options = undefined) => {
  const spawned = spawn(command, args, options);

  return new Promise((resolve) => {
    spawned.stdout.on("data", (data) => {
      console.info(chalk.bgBlack.white(data.toString()));
    });

    spawned.stderr.on("data", (data) => {
      logError(data.toString());
    });

    spawned.on("close", () => {
      resolve();
    });
  });
};

// Worker Function
const createBasicExpressAPI = () => {
  const repoEndpoint = `BhaveshAdosphere/api-ts-template`;

  // Validation - Directory Naming
  if (!flags.d || flags.d.match(/[<>:"\/\\|?*\x00-\x1F]/)) {
    logError(`Error: Missing or Invalid directory name: "${flags.d || ""}"`);
    args.showHelp();
    process.exit(-1);
  }

  // Validation - Directory Existance
  if (fs.existsSync(flags.d)) {
    logError(`Error: Directory "${flags.d}" already exists.`);
    process.exit(-1);
  }

  // Repository Cloning
  degit(repoEndpoint, {
    force: true,
    verbose: true,
  })
    .clone(flags.d)
    .then(async () => {
      console.info(chalk.bgBlack.cyan("Installing dependencies..."));
      const commandForPNPM = /^win/.test(process.platform) ? "npm.cmd" : "npm";
      const commandForPackages = /^win/.test(process.platform)
        ? "npm.cmd"
        : "npm";
      await runCommand(commandForPNPM, ["install", "-g", "npm"], {
        cwd: `${process.cwd()}/${flags.d}`,
      });
      await runCommand(commandForPackages, ["install"], {
        cwd: `${process.cwd()}/${flags.d}`,
      });
      console.info(chalk.bgBlack.cyan("Done! 🏁"));
      console.info("");
      console.info(chalk.bgBlack.white("To get started:"));
      console.info(chalk.bgBlack.cyan(`cd ${flags.d}`));
      console.info(chalk.bgBlack.cyan("npm run dev"));
    });
};

// Execution
createBasicExpressAPI();
